clear all
close all
clc

%Evaluate the nonlinearities
N = 500;
x = linspace(-1,1,N)';

y1 = nl1(x);
y2 = nl2(x);
y3 = nl3(x);

y = y1;

figure, hold on, grid on
plot(x,y)
xlabel('x[n]')
ylabel('y[n]')

%% First guesses by hand
%a)
%just a first guess
K = 4;
mu_i = linspace(-1,1,K);
Sigma_i = 0.1 * ones(1,K);

[alpha,kcond,rbf_f,yhat]=rbf(x,y,mu_i,Sigma_i);

plot(x, yhat, 'r--')

%Evaluate approximation
Q2 = q(y, yhat)

%% A more educated guess
mu_i = [-.2986 -.02 .02 .2946]';
Sigma_i = [.23 .075 .075 .23]';

[alpha,kcond,rbf_f,yhat]=rbf(x,y,mu_i,Sigma_i);

plot(x,yhat,'k--')
Q2 = q(y, yhat)

%% Sweep, grid search 
%Sweep over number of kernels and widths
N = 60;
K_max = 40;
sigma = linspace(.01, 5, N);
sigma_scaled = zeros(1,length(sigma));
K = 1:K_max;
kcond = zeros(N,N);
Q2 = zeros(K_max,N);

for i = 1:K_max
  for j = 1:N
    mu_i = linspace(-1,1,K(i));
    %Scaled with number of Kernels
    Sigma_i = ones(size(mu_i))* sigma(j)/K(i);
    %Unscaled
    %Sigma_i = ones(size(mu_i))* sigma(j);
    sigma_scaled(j) = Sigma_i(1);
    [alpha,kcond(i),rbf_f,yhat]=rbf(x,y,mu_i,Sigma_i);
    Q2(i,j) = q(y, yhat);
  end
end

figure
surface(sigma_scaled, K, Q2, 'FaceColor', 'interp')
grid on
xlabel('Sigma')
ylabel('K')
zlabel('Q2')

%Take a value from grid-search plot (i.e. find the minimum of the grid
%search)
mu_i = linspace(-1, 1, 40)';
Sigma_i = ones(1,40) * 0.03831;
[alpha,kcond,rbf_f,yhat]=rbf(x,y,mu_i,Sigma_i);

figure, hold on, grid on
plot(x,y)
plot(x,yhat,'k--')
xlabel('x[n]')
ylabel('y[n]')
Q2 = q(y, yhat)

%% Conditioning as a function of kernel number
%b)
figure, plot(x,y), grid on, hold on

num_kernels = 4:100;
kcond = zeros(length(num_kernels),1);

%colorvec = ['r','g','m','c','k'];
for i = 1:length(num_kernels);
  K = num_kernels(i);
  mu_i = linspace(-1,1,K)';
  Sigma_i = ones(K,1)*0.06;
  [alpha,kcond(i),rbf_f,yhat]=rbf(x,y,mu_i,Sigma_i);
  %plot(x,yhat, colorvec(i));
  Q2 = q(y, yhat);
end

figure
semilogy(num_kernels, kcond);
xlabel('K')
ylabel('log(condition number)')


%% Conditioning as a function of width
K = 5;
sigma = linspace(.01, 5, 100);
kcond = zeros(length(sigma),1);
mu_i = linspace(-1,1,K)';
for i = 1:length(sigma);
  Sigma_i = ones(K,1)*sigma(i);
  [alpha,kcond(i),rbf_f,yhat]=rbf(x,y,mu_i,Sigma_i);
  %plot(x,yhat, colorvec(i));
  Q2 = q(y, yhat);
end

figure
semilogy(sigma, kcond);
xlabel('\sigma')
ylabel('log(condition number)')





%% New
% K = 4;
% mu_i = linspace(-1,1,K);
% Sigma_i = 0.1 * ones(1,K);
% 
% [alpha,kcon,multiquad_f,yhat]=rbf_multiquad(x,y,mu_i,Sigma_i);
% 
% figure, hold on, grid on
% 
% plot(x, yhat, 'r--')
% 
% %Evaluate approximation
% Q2 = q(y, yhat)
% 



